# =============================================================================================
# Copyright 2018 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================================

import os
import gzip
from numpy import unique
from datetime import datetime
from pandas import read_csv
from requests import get

LANDSAT_METADATA_URL = 'http://storage.googleapis.com/gcp-public-data-landsat/index.csv.gz'
ZIP_PATH = 'index.csv.gz'

SCENES = os.path.dirname(__file__).replace('landsat', 'scenes')

if not os.path.isdir(SCENES):
    os.mkdir(SCENES)
os.chdir(SCENES)
fmt = '%Y%m%d'
date = datetime.strftime(datetime.now(), fmt)
LATEST = 'scenes_{}'.format(date)

PARSE_DATES = ['DATE_ACQUIRED', 'SENSING_TIME']


def update():
    download_latest_metadata()
    split_list()
    return None


def download_latest_metadata():
    if not os.path.isfile(ZIP_PATH):
        req = get(LANDSAT_METADATA_URL, stream=True)
        if req.status_code != 200:
            raise ValueError('Bad response {} from request.'.format(req.status_code))

        with open(ZIP_PATH, 'wb') as f:
            print('Downloading {}'.format(LANDSAT_METADATA_URL))
            for chunk in req.iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)

    if not os.path.isfile(LATEST):
        with gzip.open(ZIP_PATH, 'rb') as infile:
            print('unzipping {}'.format(ZIP_PATH))
            with open(LATEST, 'wb') as outfile:
                for line in infile:
                    outfile.write(line)

    return None


def split_list(_list=LATEST):
    csv = read_csv(_list, parse_dates=PARSE_DATES)
    sats = unique(csv.SPACECRAFT_ID).tolist()
    for sat in sats:
        df = csv[csv.SPACECRAFT_ID == sat]
        df.to_pickle(sat)

    return None


if __name__ == '__main__':

    update()
# ========================= EOF ================================================================



# =============================================================================================
# Copyright 2018 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# =============================================================================================

import os


class BandMap:
    def __init__(self):
        pass

    @property
    def selected(self):
        band_mapping = {'LC8': ['1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11'],
                        'LE7': ['1', '2', '3', '4', '5', '6_VCID_1', '6_VCID_2', '7', '8'],
                        'LT5': ['1', '2', '3', '4', '5', '6', '7']}
        return band_mapping

    @property
    def file_suffixes(self):
        b = {'LANDSAT_1': ['B1.TIF', 'B2.TIF', 'B3.TIF', 'B4.TIF', 'B5.TIF', 'B6.TIF', 'B7.TIF', 'MTL.txt'],
             'LANDSAT_2': ['B1.TIF', 'B2.TIF', 'B3.TIF', 'B4.TIF', 'B5.TIF', 'B6.TIF', 'B7.TIF', 'MTL.txt'],
             'LANDSAT_3': ['B1.TIF', 'B2.TIF', 'B3.TIF', 'B4.TIF', 'B5.TIF', 'B6.TIF', 'B7.TIF', 'MTL.txt'],
             'LANDSAT_4': ['B1.TIF', 'B2.TIF', 'B3.TIF', 'B4.TIF', 'B5.TIF', 'B6.TIF', 'B7.TIF', 'MTL.txt'],
             'LANDSAT_5': ['B1.TIF', 'B2.TIF', 'B3.TIF', 'B4.TIF', 'B5.TIF', 'B6.TIF', 'B7.TIF', 'MTL.txt'],

             'LANDSAT_7': ['B1.TIF', 'B2.TIF', 'B3.TIF', 'B4.TIF', 'B5.TIF',
                           'B6_VCID_1.TIF', 'B6_VCID_2.TIF', 'B7.TIF', 'B8.TIF', 'MTL.txt'],

             'LANDSAT_8': ['B1.TIF', 'B2.TIF', 'B3.TIF', 'B4.TIF', 'B5.TIF', 'B6.TIF',
                           'B7.TIF', 'B8.TIF', 'B9.TIF', 'B10.TIF', 'B11.TIF', 'BQA.TIF', 'MTL.txt']}
        return b


if __name__ == '__main__':
    home = os.path.expanduser('~')


# ========================= EOF ================================================================



# ===============================================================================
# Copyright 2017 ross
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================
from traits.api import HasTraits, Enum, Date, Button
from traitsui.api import View, Item, UItem, Controller

from .download_composer import download_landsat


# here is a gui down in a true MVC pattern.  See passed EOF for shortened form
class LandsatDownloadModel(HasTraits):
    satellite = Enum('LT5', 'LE7', 'LC8')
    start_date = Date
    end_date = Date

    def do_download(self):
        download_landsat(self.start_date, self.end_date, self.satellite)


class LandsatController(Controller):
    download_button = Button

    def _download_button_fired(self):
        self.model.do_download()


LANDSAT_VIEW = View(Item('start_date', style='custom'),
                    Item('end_date', style='custom'),
                    Item('satellite'),
                    Item('controller.download_button'),
                    title='Configure Landsat Downloader',
                    buttons=['OK', 'Cancel'])

if __name__ == '__main__':
    m = LandsatDownloadModel()
    c = LandsatController(model=m)

    c.configure_traits(view=LANDSAT_VIEW)

# ============= EOF =============================================
#
# class Downloader(HasTraits):
#     satellite = Enum('LT5', 'LE7', 'LC8')
#     start_date = Date
#     end_date = Date
#     download_button = Button
#
#     def do_download(self):
#         download_landsat(self.start_date, self.end_date, self.satellite)
#
#     def _download_button_fired(self):
#         self.do_download()
#
#     def traits_view(self):
#         v = View(Item('start_date', style='custom'),
#                  Item('end_date', style='custom'),
#                  Item('satellite'),
#                  Item('download_button'),
#                  title='Configure Landsat Downloader',
#                  buttons=['OK', 'Cancel'])
#
#         return v
#
#
# if __name__ == '__main__':
#     d = Downloader()
#     d.configure_traits()


#!/usr/bin/env python
# ===============================================================================
# Copyright 2017 dgketchum
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ===============================================================================
import os
import argparse
import sys
import yaml

from google_download import GoogleDownload
from pymetric_download import download
from update_landsat_metadata import update


class TooFewInputsError(Exception):
    pass


DEFAULT_CFG = '''
# date format: 'YYYY-MM-DD'
start: '2007-05-01'
end: '2007-05-31'
path: 43
row: 30
latitude:
longitude:
output_path: None
satellite: 5

return_list: False
zipped: True
max_cloud_percent: 100

# pymetric directory structure: e.g. D:/pyMETRIC/harney/landsat/path/row/year
# using pymetric_root and clear_scenes overrides all other arguments
# leave both blank to disable

pymetric_root: D:/pyMETRIC/root
clear_scenes: D:/pyMETRIC/misc/clear_scenes.txt
'''

CONFIG_PLACEMENT = os.path.dirname(__file__)


def create_parser():
    parser = argparse.ArgumentParser(prog='landsat', description='Download and unzip landsat data.')

    parser.add_argument('--satellite', '-sat', help='Satellite number: 1-8, except 6', type=int)
    parser.add_argument('--start', help='Start date in format YYYY-MM-DD', type=str)
    parser.add_argument('--end', help='End date in format YYYY-MM-DD', type=str)
    parser.add_argument('-lat', '--latitude', help='Latitude, decimal degrees', type=float, default=None)
    parser.add_argument('-lon', '--longitude', help='Longitude, decimal degrees', type=float, default=None)
    parser.add_argument('-p', '--path', help='The path', type=str, default=None)
    parser.add_argument('-r', '--row', help='The row', type=str, default=None)
    parser.add_argument('-o', '--output-path', help='Output directory', default=CONFIG_PLACEMENT)

    parser.add_argument('-conf', '--configuration', help='Path to your configuration file. If a directory is provided,'
                                                         'a template cofiguration file will be created there.')
    parser.add_argument('-cs', '--clear-scenes', help='Path to your clear scenes file.')
    parser.add_argument('-pym', '--pymetric-root', help='Path to your pyMETRIC study area root dir.')

    parser.add_argument('--return-list', help='Just return list of images without downloading', action='store_true',
                        default=False)

    parser.add_argument('--zipped', help='Download .tar.gz file(s), without unzipping',
                        action='store_true', default=False)

    parser.add_argument('--max-cloud-percent', help='Maximum percent of of image obscured by clouds accepted,'
                                                    ' type integer',
                        default=100)

    parser.add_argument('--update-scenes', help='Update the scenes list this program uses to discover the '
                                                'latest imagery.', default=False)

    return parser


def main(args):
    return_scene_list = False

    if args:

        cfg = {}
        for arg in vars(args):
            var = getattr(args, arg)
            if var is not None:
                cfg[arg] = var

        if cfg['update_scenes']:
            update()

        if args.configuration:
            if os.path.isdir(args.configuration):
                print('Creating template configuration file at {}.'.format(args.configuration))
                check_config(args.configuration)

            with open(args.configuration, 'r') as rfile:
                ycfg = yaml.load(rfile)
                cfg.update(ycfg)

            if cfg['return_list']:
                return_scene_list = True

            del cfg['return_list']
            del cfg['configuration']
            del cfg['update_scenes']

            if cfg['pymetric_root']:
                download(cfg['clear_scenes'], cfg['pymetric_root'])
            else:
                del cfg['clear_scenes']
                del cfg['pymetric_root']

                g = GoogleDownload(**cfg)
                if return_scene_list:
                    return g.candidate_scenes(return_list=True)
                g.download()

        else:
            del cfg['return_list']
            del cfg['update_scenes']

            g = GoogleDownload(**cfg)
            if return_scene_list:
                return g.candidate_scenes(return_list=True)
            else:
                g.download()


def cli_runner():
    parser = create_parser()
    args = parser.parse_args()
    if len(sys.argv) == 1:
        parser.print_help(sys.stderr)
        sys.exit(1)
    return main(args)


def check_config(dirname):
    path = os.path.join(dirname, 'downloader_config.yml')
    print('\n*****A default config file {} will be written'.format(path))

    with open(path, 'w') as wfile:
        print('-------------- DEFAULT CONFIG -----------------')
        print(DEFAULT_CFG)
        print('-----------------------------------------------')
        wfile.write(DEFAULT_CFG)

    print('***** Please edit the config file at {} and run the downer again *****\n'.format(
        dirname))

    sys.exit()


if __name__ == '__main__':
    cli_runner()
# ===============================================================================



import os
from datetime import datetime

from . import download_composer

if __name__ == '__main__':
    home = os.path.expanduser('~')
    start = datetime(2015, 6, 25)
    end = datetime(2015, 8, 2)
    satellite = 'LE7'
    output = os.path.join(home, 'images', 'sandbox', 'nonexistent')
    usgs_creds = os.path.join(home, 'images', 'usgs.txt')
    path = 43
    row = 33
    # latitude = 45.6
    # longitude = -107.9
    download_composer.download_landsat(start, end, satellite, path=path, row=row, output_path=output)

# ===============================================================================
